<?php

namespace Slack\Blockkit\Blocks\Elements;

use \Exception;
use Illuminate\Support\Collection;
use Slack\BlockKit\Blocks;
use Slack\Blockkit\Element;

final class MultiStaticSelect extends Element
{
	protected const LIMITS = [
		'action_id' => 255,
		'placeholder' => 150,
	];

	private const MAX_OPTIONS = 100;

	/**
	 * @param Text $placeholder
	 * @param string $action_id
	 * @param Collection $options
	 * @throws Exception
	 * @todo We dont handle option_groups yet.
	 */
	public function __construct(Text $placeholder,string $action_id,Collection $options)
	{
		parent::__construct();

		// Defaults
		$this->type = 'multi_static_select';

		if ($placeholder->type != 'plain_text')
			throw new Exception(sprintf('Text must be plain_text not %s',$placeholder->type));

		if (strlen($placeholder->text) > self::LIMITS['placeholder'])
			throw new Exception(sprintf('Text must be %d chars or less',self::LIMITS['placeholder']));

		$this->placeholder = $placeholder;

		$this->action_id = $this->validate('action_id',$action_id);

		if (count($options) > self::MAX_OPTIONS)
			throw new Exception(sprintf('Can only have maximum %d options',self::MAX_OPTIONS));

		$this->options = $options->transform(function($item) {
			return ['text'=>Text::item($item->name,'plain_text'),'value'=>(string)$item->id];
		});
	}

	public static function item(Text $placeholder,string $action_id,Collection $options): self
	{
		return new self($placeholder,$action_id,$options);
	}

	/* OPTIONAL ITEMS */

	public function confirm(Confirm $confirm): self
	{
		$this->confirm = $confirm;

		return $this;
	}

	public function focus_on_load(bool $bool): self
	{
		$this->focus_on_load = $bool ? 'true' : 'false';

		return $this;
	}

	public function initial_options(Collection $collection): self
	{
		// No initial options.
		if (! count($collection))
			return $this;

		if (count($collection) > self::MAX_OPTIONS)
			throw new Exception(sprintf('Can only have maximum %d options',self::MAX_OPTIONS));

		$this->initial_options = $collection->transform(function($item) {
			return ['text'=>Text::item($item->name,'plain_text'),'value'=>(string)$item->id];
		});

		return $this;
	}

	public function max_selected_items(int $int): self
	{
		if ($int < 1)
			throw new Exception('Minimum 1 options must be configured');

		$this->max_selected_items = $int;

		return $this;
	}
}